import 'package:dio/dio.dart';

import 'package:vivanta_connect_flutter/config/config.dart';
import 'package:vivanta_connect_flutter/data/brand.dart';
import 'package:vivanta_connect_flutter/data/client.dart';
import 'package:vivanta_connect_flutter/data/user.dart';

class Http {
  final dio = Dio();

  Future<Client?> me(
    String apiKey,
    String customerId,
  ) async {
    try {
      final result = await dio.get(
        '${Config.url}/clients/me',
        options: Options(
          headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            "x-api-key": apiKey,
            "x-customer": customerId,
          },
        ),
      );

      return Client.fromJson(result.data);
    } catch (e) {
      return null;
    }
  }

  Future<User?> authenticateUser(
    String apiKey,
    String customerId,
    String externalUserId,
    String companyId,
  ) async {
    try {
      final result = await dio.post(
        '${Config.url}/users/auth',
        data: {
          'externalUserId': externalUserId,
          'externalCompanyId': companyId,
        },
        options: Options(
          headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            "x-api-key": apiKey,
            "x-customer": customerId,
          },
        ),
      );

      return User.fromJson(result.data);
    } catch (e) {
      return null;
    }
  }

  Future<List<Brand>> getBrands(String token) async {
    try {
      final result = await dio.get<List<dynamic>>(
        '${Config.url}/brands/user/all',
        options: Options(
          headers: {
            'Authorization': 'Bearer $token',
          },
        ),
      );

      if (result.data != null) {
        return result.data!
            .map((e) => Brand.fromJson(e as Map<String, dynamic>))
            .toList();
      }
    } catch (e) {
      return [];
    }
    return [];
  }

  Future<String?> getEmbeddedGraphResource(
    String apiKey,
    String customerId,
    String externalUserId,
    String type, {
    int? brandId,
  }) async {
    try {
      final payload = {
        'useExternal': true,
        'type': type,
      };

      if (brandId != null) {
        payload['brandId'] = brandId;
      }

      final result = await dio.post(
        '${Config.url}/graphs/$externalUserId/link',
        data: payload,
        options: Options(
          headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            "x-api-key": apiKey,
            "x-customer": customerId,
          },
        ),
      );
      return result.data['link'];
    } catch (e) {
      return null;
    }
  }

  Future<bool> uploadAppleHealthData(
    String apiKey,
    String customerId,
    String externalUserId,
    String data,
  ) async {
    try {
      final payload = {
        'externalUserId': externalUserId,
        'data': [data]
      };
      await dio.post(
        '${Config.url}/custom-data/apple-health/upload',
        data: payload,
        options: Options(
          headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            "x-api-key": apiKey,
            "x-customer": customerId,
          },
        ),
      );

      return true;
    } catch (e) {
      return false;
    }
  }

  Future<bool> uploadAppleHealthArrayData(
    String apiKey,
    String customerId,
    String externalUserId,
    List<String> data,
  ) async {
    try {
      final payload = {'externalUserId': externalUserId, 'data': data};
      await dio.post(
        '${Config.url}/custom-data/apple-health/upload',
        data: payload,
        options: Options(
          headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            "x-api-key": apiKey,
            "x-customer": customerId,
          },
        ),
      );

      return true;
    } catch (e) {
      return false;
    }
  }

  Future<bool> uploadGoogleHealthConnectData(
    String apiKey,
    String customerId,
    String externalUserId,
    String data,
  ) async {
    try {
      final payload = {
        'externalUserId': externalUserId,
        'data': [data]
      };
      await dio.post(
        '${Config.url}/custom-data/google-health/upload',
        data: payload,
        options: Options(
          headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            "x-api-key": apiKey,
            "x-customer": customerId,
          },
        ),
      );

      return true;
    } catch (e) {
      return false;
    }
  }

  Future<bool> uploadGoogleHealthConnectArrayData(
    String apiKey,
    String customerId,
    String externalUserId,
    List<String> data,
  ) async {
    try {
      final payload = {'externalUserId': externalUserId, 'data': data};
      await dio.post(
        '${Config.url}/custom-data/google-health/upload',
        data: payload,
        options: Options(
          headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            "x-api-key": apiKey,
            "x-customer": customerId,
          },
        ),
      );

      return true;
    } catch (e) {
      return false;
    }
  }
}
